@@ -1,24 +1,3 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef CLUSTER_H
 #define CLUSTER_H
 
@@ -79,6 +58,7 @@ namespace torali
   // Edge struct
   template<typename TWeight, typename TVertex>
   struct EdgeRecord {
+    typedef TVertex TVertexType;
     TVertex source;
     TVertex target;
     TWeight weight;
@@ -95,31 +75,6 @@ namespace torali
     }
   };
 
-
-
-
-  
-  // Edge struct
-  template<typename TWeight, typename TVertex>
-  struct SREdgeRecord {
-    typedef TVertex TVertexType;
-    TVertex source;
-    TVertex target;
-    TWeight weight;
-
-    SREdgeRecord(TVertex s, TVertex t, TWeight w) : source(s), target(t), weight(w) {}
-  };
-
-  // Sort edge records
-  template<typename TRecord>
-  struct SortSREdgeRecords : public std::binary_function<TRecord, TRecord, bool>
-  {
-    inline bool operator()(TRecord const& e1, TRecord const& e2) const {
-      return ((e1.weight < e2.weight) || ((e1.weight == e2.weight) && (e1.source < e2.source)) || ((e1.weight == e2.weight) && (e1.source == e2.source) && (e1.target < e2.target)));
-    }
-  };
-
-
   // Initialize clique, deletions
   template<typename TBamRecord, typename TSize>
   inline void
@@ -270,17 +225,20 @@ namespace torali
     // Iterate all components
     for(typename TCompEdgeList::iterator compIt = compEdge.begin(); compIt != compEdge.end(); ++compIt) {
       // Sort edges by weight
-      std::sort(compIt->second.begin(), compIt->second.end(), SortSREdgeRecords<TEdgeRecord>());
+      std::sort(compIt->second.begin(), compIt->second.end(), SortEdgeRecords<TEdgeRecord>());
 
       // Find a large clique
       typename TEdgeList::const_iterator itWEdge = compIt->second.begin();
       typename TEdgeList::const_iterator itWEdgeEnd = compIt->second.end();
       typedef std::set<TVertex> TCliqueMembers;
+      typedef std::set<std::size_t> TSeeds;
       TCliqueMembers clique;
       TCliqueMembers incompatible;
+      TSeeds seeds;
       
       // Initialize clique
       clique.insert(itWEdge->source);
+      seeds.insert(br[itWEdge->source].id);
       int32_t chr = br[itWEdge->source].chr;
       int32_t chr2 = br[itWEdge->source].chr2;
       int32_t ciposlow = br[itWEdge->source].pos;
@@ -289,6 +247,7 @@ namespace torali
       int32_t ciendlow = br[itWEdge->source].pos2;
       uint64_t pos2 = br[itWEdge->source].pos2;
       int32_t ciendhigh = br[itWEdge->source].pos2;
+      int32_t mapq = br[itWEdge->source].qual;
       int32_t inslen = br[itWEdge->source].inslen;
 
       // Grow clique
@@ -303,6 +262,7 @@ namespace torali
 	  else if ((clique.find(itWEdge->source) != clique.end()) && (clique.find(itWEdge->target) == clique.end())) v = itWEdge->target;
 	  else continue;
 	  if (incompatible.find(v) != incompatible.end()) continue;
+	  if (seeds.find(br[v].id) != seeds.end()) continue;
 	  // Try to update clique with this vertex
 	  int32_t newCiPosLow = std::min(br[v].pos, ciposlow);
 	  int32_t newCiPosHigh = std::max(br[v].pos, ciposhigh);
@@ -312,12 +272,14 @@ namespace torali
 	  if (cliqueGrow) {
 	    // Accept new vertex
 	    clique.insert(v);
+	    seeds.insert(br[v].id);
 	    ciposlow = newCiPosLow;
 	    pos += br[v].pos;
 	    ciposhigh = newCiPosHigh;
 	    ciendlow = newCiEndLow;
 	    pos2 += br[v].pos2;
 	    ciendhigh = newCiEndHigh;
+	    mapq += br[v].qual;
 	    inslen += br[v].inslen;
 	  } else incompatible.insert(v);
 	}
@@ -332,9 +294,12 @@ namespace torali
 	  std::cerr << "Warning: Confidence intervals out of bounds: " << ciposlow << ',' << svStart << ',' << ciposhigh << ':' << ciendlow << ',' << svEnd << ',' << ciendhigh << std::endl;
 	}
 	int32_t svid = sv.size();
-	sv.push_back(StructuralVariantRecord(chr, svStart, chr2, svEnd, (ciposlow - svStart), (ciposhigh - svStart), (ciendlow - svEnd), (ciendhigh - svEnd), clique.size(), svInsLen, svt, svid));
+	sv.push_back(StructuralVariantRecord(chr, svStart, chr2, svEnd, (ciposlow - svStart), (ciposhigh - svStart), (ciendlow - svEnd), (ciendhigh - svEnd), clique.size(), mapq, svInsLen, svt, svid));
 	// Reads assigned
-	for(typename TCliqueMembers::iterator itC = clique.begin(); itC != clique.end(); ++itC) br[*itC].svid = svid;
+	for(typename TCliqueMembers::iterator itC = clique.begin(); itC != clique.end(); ++itC) {
+	  //std::cerr << svid << ',' << br[*itC].id << std::endl;
+	  br[*itC].svid = svid;
+	}
       }
     }
   }
@@ -355,7 +320,7 @@ namespace torali
       // Edge lists for each component
       typedef uint32_t TWeightType;
       typedef uint32_t TVertex;
-      typedef SREdgeRecord<TWeightType, TVertex> TEdgeRecord;
+      typedef EdgeRecord<TWeightType, TVertex> TEdgeRecord;
       typedef std::vector<TEdgeRecord> TEdgeList;
       typedef std::map<uint32_t, TEdgeList> TCompEdgeList;
       TCompEdgeList compEdge;
@@ -381,6 +346,7 @@ namespace torali
 	  for(uint32_t j = i + 1; j<br.size(); ++j) {
 	    if (br[j].chr == refIdx) {
 	      if ( (uint32_t) (br[j].pos - br[i].pos) > varisize) break;
+	      if ((svt == 4) && (std::abs(br[j].inslen - br[i].inslen) > varisize)) continue;
 	      if ( (uint32_t) std::abs(br[j].pos2 - br[i].pos2) < varisize) {
 		// Update last connected node
 		if (j > lastConnectedNode) lastConnectedNode = j;
@@ -532,8 +498,9 @@ namespace torali
     uint32_t numComp = 0;
       
     // Edge lists for each component
-    typedef uint8_t TWeightType;
-    typedef EdgeRecord<TWeightType, std::size_t> TEdgeRecord;
+    typedef uint32_t TWeightType;
+    typedef uint32_t TVertex;
+    typedef EdgeRecord<TWeightType, TVertex> TEdgeRecord;
     typedef std::vector<TEdgeRecord> TEdgeList;
     typedef std::map<uint32_t, TEdgeList> TCompEdgeList;
     TCompEdgeList compEdge;
@@ -557,7 +524,7 @@ namespace torali
       TBamRecord::const_iterator bamItNext = bamIt;
       ++bamItNext;
       std::size_t bamItIndexNext = bamItIndex + 1;
-      for(; ((bamItNext != bamRecord.end()) && (abs(_minCoord(bamItNext->pos, bamItNext->mpos, svt) + bamItNext->alen - minCoord) <= varisize)) ; ++bamItNext, ++bamItIndexNext) {
+      for(; ((bamItNext != bamRecord.end()) && ((uint32_t) std::abs(_minCoord(bamItNext->pos, bamItNext->mpos, svt) + bamItNext->alen - minCoord) <= varisize)) ; ++bamItNext, ++bamItIndexNext) {
 	  // Check that mate chr agree (only for translocations)
 	if (bamIt->mtid != bamItNext->mtid) continue;
 	
