@@ -1,24 +1,3 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef COVERAGE_H
 #define COVERAGE_H
 
@@ -30,6 +9,7 @@ Contact: Tobias Rausch (rausch@embl.de)
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/zlib.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
+#include <boost/progress.hpp>
 
 #include <htslib/sam.h>
 
@@ -350,9 +330,9 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 
   // Dump file
   boost::iostreams::filtering_ostream dumpOut;
-  if (c.dumpflag) {
+  if (c.hasDumpFile) {
     dumpOut.push(boost::iostreams::gzip_compressor());
-    dumpOut.push(boost::iostreams::file_sink(c.srpedump.string().c_str(), std::ios_base::out | std::ios_base::binary));
+    dumpOut.push(boost::iostreams::file_sink(c.dumpfile.string().c_str(), std::ios_base::out | std::ios_base::binary));
     dumpOut << "#svid\tbam\tqname\tchr\tpos\tmatechr\tmatepos\tmapq\ttype" << std::endl;
   }
 
@@ -544,7 +524,7 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 		      uint8_t* hpptr = bam_aux_get(rec, "HP");
 #pragma omp critical
 		      {
-			if (c.dumpflag) {
+			if (c.hasDumpFile) {
 			  std::string svid(_addID(itBp->svt));
 			  std::string padNumber = boost::lexical_cast<std::string>(itBp->id);
 			  padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
@@ -687,7 +667,7 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 		  uint8_t* hpptr = bam_aux_get(rec, "HP");
 #pragma omp critical
 		  {
-		    if (c.dumpflag) {
+		    if (c.hasDumpFile) {
 		      std::string svid(_addID(itSpan->svt));
 		      std::string padNumber = boost::lexical_cast<std::string>(itSpan->id);
 		      padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
