@@ -1,24 +1,3 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef DELLY_H
 #define DELLY_H
 
@@ -86,7 +65,7 @@ struct Config {
   bool hasExcludeFile;
   bool hasVcfFile;
   bool isHaplotagged;
-  bool dumpflag;
+  bool hasDumpFile;
   bool svtcmd;
   std::set<int32_t> svtset;
   DnaScore<int> aliscore;
@@ -94,7 +73,7 @@ struct Config {
   boost::filesystem::path vcffile;
   boost::filesystem::path genome;
   boost::filesystem::path exclude;
-  boost::filesystem::path srpedump;
+  boost::filesystem::path dumpfile;
   std::vector<boost::filesystem::path> files;
   std::vector<std::string> sampleName;
 };
@@ -124,8 +103,7 @@ _annotateCoverage(TConfig& c, bam_hdr_t* hdr, TSampleLib& sampleLib, TSVs& svs,
     if (!validChr[refIndex]) continue;
     std::string tname(hdr->target_name[refIndex]);
     int32_t seqlen = -1;
-    char* seq = NULL;
-    seq = faidx_fetch_seq(fai, tname.c_str(), 0, hdr->target_len[refIndex], &seqlen);
+    char* seq = faidx_fetch_seq(fai, tname.c_str(), 0, hdr->target_len[refIndex], &seqlen);
     bool nrun = false;
     int nstart = seqlen;
     for(int i=0; i<seqlen; ++i) {
@@ -318,7 +296,7 @@ inline int dellyRun(TConfigStruct& c) {
   typedef std::vector<TChrIntervals> TRegionsGenome;
   TRegionsGenome validRegions;
   if (!_parseExcludeIntervals(c, hdr, validRegions)) {
-    std::cerr << "Delly couldn't parse excluce intervals!" << std::endl;
+    std::cerr << "Delly couldn't parse exclude intervals!" << std::endl;
     bam_hdr_destroy(hdr);
     sam_close(samfile);
     return 1;
@@ -443,7 +421,7 @@ int delly(int argc, char **argv) {
   geno.add_options()
     ("vcffile,v", boost::program_options::value<boost::filesystem::path>(&c.vcffile), "input VCF/BCF file for genotyping")
     ("geno-qual,u", boost::program_options::value<uint16_t>(&c.minGenoQual)->default_value(5), "min. mapping quality for genotyping")
-    ("dump,d", boost::program_options::value<boost::filesystem::path>(&c.srpedump), "gzipped output file for SV-reads (optional)")
+    ("dump,d", boost::program_options::value<boost::filesystem::path>(&c.dumpfile), "gzipped output file for SV-reads (optional)")
     ;
 
   // Define hidden options
@@ -510,8 +488,8 @@ int delly(int argc, char **argv) {
   }
   
   // Dump PE and SR support?
-  if (vm.count("dump")) c.dumpflag = true;
-  else c.dumpflag = false;
+  if (vm.count("dump")) c.hasDumpFile = true;
+  else c.hasDumpFile = false;
 
   // Check quality cuts
   if (c.minMapQual > c.minTraQual) c.minTraQual = c.minMapQual;
@@ -570,11 +548,9 @@ int delly(int argc, char **argv) {
       }
     }
     fai_destroy(fai);
-    std::string sampleName;
-    if (!getSMTag(std::string(hdr->text), c.files[file_c].stem().string(), sampleName)) {
-      std::cerr << "Only one sample (@RG:SM) is allowed per input BAM file " << c.files[file_c].string() << std::endl;
-      return 1;
-    } else c.sampleName[file_c] = sampleName;
+    std::string sampleName = "unknown";
+    getSMTag(std::string(hdr->text), c.files[file_c].stem().string(), sampleName);
+    c.sampleName[file_c] = sampleName;
     bam_hdr_destroy(hdr);
     hts_idx_destroy(idx);
     sam_close(samfile);
