@@ -1,24 +1,3 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef FILTER_H
 #define FILTER_H
 
@@ -450,6 +429,10 @@ int filter(int argc, char **argv) {
       std::cerr << "Fail to header for " << c.vcffile.string() << std::endl;
       return 1;
     }
+    if (!(bcf_hdr_nsamples(hdr)>0)) {
+      std::cerr << "BCF/VCF file has no sample genotypes!" << std::endl;
+      return 1;
+    }
     // Check sample names
     if (c.filter == "somatic") {
       for (int i = 0; i < bcf_hdr_nsamples(hdr); ++i) {
