@@ -1,24 +1,3 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef MERGE_H
 #define MERGE_H
 
@@ -31,6 +10,9 @@ Contact: Tobias Rausch (rausch@embl.de)
 #include <boost/program_options/options_description.hpp>
 #include <boost/program_options/parsers.hpp>
 #include <boost/program_options/variables_map.hpp>
+#include <boost/uuid/uuid.hpp>
+#include <boost/uuid/uuid_generators.hpp>
+#include <boost/uuid/uuid_io.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/gregorian/gregorian.hpp>
 #include <boost/iostreams/stream.hpp>
@@ -59,11 +41,14 @@ namespace torali
 struct MergeConfig {
   bool filterForPass;
   bool filterForPrecise;
+  uint32_t chunksize;
   uint32_t svcounter;
   uint32_t bpoffset;
   uint32_t minsize;
   uint32_t maxsize;
+  uint32_t coverage;
   float recoverlap;
+  float vaf;
   boost::filesystem::path outfile;
   std::vector<boost::filesystem::path> files;
 };
@@ -71,9 +56,9 @@ struct MergeConfig {
 struct IntervalScore {
   uint32_t start;
   uint32_t end;
-  uint32_t score;
+  int32_t score;
   
-  IntervalScore(uint32_t s, uint32_t e, uint32_t c) : start(s), end(e), score(c) {}
+  IntervalScore(uint32_t s, uint32_t e, int32_t c) : start(s), end(e), score(c) {}
 };
 
 template<typename TRecord>
@@ -125,10 +110,10 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
     int32_t* sr = NULL;
     int32_t nmapq = 0;
     int32_t* mapq = NULL;
+    int32_t nsrmapq = 0;
+    int32_t* srmapq = NULL;
     int32_t nct = 0;
     char* ct = NULL;
-    int32_t nsrq = 0;
-    float* srq = NULL;
     int32_t nsvt = 0;
     char* svt = NULL;
     while (bcf_read(ifile, hdr, rec) == 0) {
@@ -164,20 +149,63 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
       if (bcf_get_info_flag(hdr, rec, "PRECISE", 0, 0) > 0) precise=true;
       if ((c.filterForPrecise) && (!precise)) continue;
 
-      unsigned int peSupport = 0;
-      if (bcf_get_info_int32(hdr, rec, "PE", &pe, &npe) > 0) peSupport = *pe;
-      unsigned int srSupport = 0;
-      if (bcf_get_info_int32(hdr, rec, "SR", &sr, &nsr) > 0) srSupport = *sr;
-      // Remove this line
-      //if (srSupport > 0) precise = true;
+      // Variant allele frequency filter
+      if ((c.vaf > 0) || (c.coverage > 0)) {
+	float maxvaf = 0;
+	uint32_t maxcov = 0;
+	bcf_unpack(rec, BCF_UN_ALL);
+	int ndv = 0;
+	int32_t* dv = NULL;
+	int ndr = 0;
+	int32_t* dr = NULL;
+	int nrv = 0;
+	int32_t* rv = NULL;
+	int nrr = 0;
+	int32_t* rr = NULL;
+	int ngt = 0;
+	int32_t* gt = NULL;
+	bcf_get_format_int32(hdr, rec, "DV", &dv, &ndv);
+	bcf_get_format_int32(hdr, rec, "DR", &dr, &ndr);
+	bcf_get_format_int32(hdr, rec, "RV", &rv, &nrv);
+	bcf_get_format_int32(hdr, rec, "RR", &rr, &nrr);
+	bcf_get_format_int32(hdr, rec, "GT", &gt, &ngt);
+	for(int32_t i = 0; i < bcf_hdr_nsamples(hdr); ++i) {
+	  if ((bcf_gt_allele(gt[i*2]) != -1) && (bcf_gt_allele(gt[i*2 + 1]) != -1)) {
+	    uint32_t supportsum = 0;
+	    if (precise) supportsum = rr[i] + rv[i];
+	    else supportsum = dr[i] + dv[i];
+	    if (supportsum > 0) {
+	      double vaf = 0;
+	      if (precise) vaf = (double) rv[i] / (double) supportsum;
+	      else vaf = (double) dv[i] / (double) supportsum;
+	      if (vaf > maxvaf) maxvaf = vaf;
+	      if (supportsum > maxcov) maxcov = supportsum; 
+	    }
+	  }
+	}
+	// Debug
+	//std::cerr << maxcov << '\t' << maxvaf << std::endl;
+	if (dv != NULL) free(dv);
+	if (dr != NULL) free(dr);
+	if (rv != NULL) free(rv);
+	if (rr != NULL) free(rr);
+	if (gt != NULL) free(gt);
+	if ((maxvaf < c.vaf) || (maxcov < c.coverage)) continue;
+      }
 
-      uint8_t peMapQuality = 0;
-      if (bcf_get_info_int32(hdr, rec, "MAPQ", &mapq, &nmapq) > 0) peMapQuality = (uint8_t) *mapq;
-      float srAlignQuality = 0;
-      if (bcf_get_info_float(hdr, rec, "SRQ", &srq, &nsrq) > 0) srAlignQuality = *srq;
+      // Get SV support
+      int32_t peSupport = 0;
+      if (bcf_get_info_int32(hdr, rec, "PE", &pe, &npe) > 0) peSupport = *pe;
+      int32_t srSupport = 0;
+      if (bcf_get_info_int32(hdr, rec, "SR", &sr, &nsr) > 0) srSupport = *sr;
+
+      int32_t peMapQuality = 0;
+      if (bcf_get_info_int32(hdr, rec, "MAPQ", &mapq, &nmapq) > 0) peMapQuality = *mapq;
+      int32_t srMapQuality = 0;
+      if (bcf_get_info_int32(hdr, rec, "SRMAPQ", &srmapq, &nsrmapq) > 0) srMapQuality = *srmapq;
 
       // Quality score for the SV
-      uint32_t score = 0;
+      int32_t score = 3 * srSupport * srMapQuality + peSupport * peMapQuality;
       if (_isKeyPresent(hdr, "SCORE")) {
 	int32_t nvcfscore = 0;
 	if (_getInfoType(hdr, "SCORE") == BCF_HT_INT) {
@@ -191,9 +219,6 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
 	  score = (uint32_t) (*vcfscore * 10000); // for scores in [0,1] 
 	  free(vcfscore);
 	}
-      } else {
-	if (precise) score = srSupport * (uint32_t) (100 * srAlignQuality);
-	else score = peSupport * (uint32_t) peMapQuality;
       }
 
       // Store the interval
@@ -205,7 +230,7 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
     if (sr != NULL) free(sr);
     if (mapq != NULL) free(mapq);
     if (ct != NULL) free(ct);
-    if (srq != NULL) free(srq);
+    if (srmapq != NULL) free(srmapq);
     if (svt != NULL) free(svt);
     bcf_hdr_destroy(hdr);
     bcf_close(ifile);
@@ -284,6 +309,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   bcf_hdr_append(hdr_out, "##INFO=<ID=PE,Number=1,Type=Integer,Description=\"Paired-end support of the structural variant\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=MAPQ,Number=1,Type=Integer,Description=\"Median mapping quality of paired-ends\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Split-read support\">");
+  bcf_hdr_append(hdr_out, "##INFO=<ID=SRMAPQ,Number=1,Type=Integer,Description=\"Median mapping quality of split-reads\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SRQ,Number=1,Type=Float,Description=\"Split-read consensus alignment quality\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
@@ -350,10 +376,12 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   int32_t* homlen = NULL;
   int32_t nmapq = 0;
   int32_t* mapq = NULL;
-  int32_t nct = 0;
-  char* ct = NULL;
+  int32_t nsrmapq = 0;
+  int32_t* srmapq = NULL;
   int32_t nsrq = 0;
   float* srq = NULL;
+  int32_t nct = 0;
+  char* ct = NULL;
   int32_t nsvt = 0;
   char* svt = NULL;
   int32_t nchr2 = 0;
@@ -409,18 +437,18 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	  // Remove this line
 	  //if (srSupport > 0) precise = true;
 	  
-	  uint8_t peMapQuality = 0;
-	  if (bcf_get_info_int32(hdr[idx], rec[idx], "MAPQ", &mapq, &nmapq) > 0) peMapQuality = (uint8_t) *mapq;
-	  float srAlignQuality = 0;
-	  if (bcf_get_info_float(hdr[idx], rec[idx], "SRQ", &srq, &nsrq) > 0) srAlignQuality = *srq;
+	  int32_t peMapQuality = 0;
+	  if (bcf_get_info_int32(hdr[idx], rec[idx], "MAPQ", &mapq, &nmapq) > 0) peMapQuality = *mapq;
+	  int32_t srMapQuality = 0;
+	  if (bcf_get_info_int32(hdr[idx], rec[idx], "SRMAPQ", &srmapq, &nsrmapq) > 0) srMapQuality = *srmapq;
 	  std::string chr2Name = chrName;
 	  if (bcf_get_info_string(hdr[idx], rec[idx], "CHR2", &chr2, &nchr2) > 0) {
 	    chr2Name = std::string(chr2);
 	    //mtid = cMap[chr2Name];
 	  }
 
-	  // Proxy quality score for the SV
-	  uint32_t score = 0;
+	  // Quality score for the SV
+	  int32_t score = 3 * srSupport * srMapQuality + peSupport * peMapQuality;
 	  if (_isKeyPresent(hdr[idx], "SCORE")) {
 	    int32_t nvcfscore = 0;
 	    if (_getInfoType(hdr[idx], "SCORE") == BCF_HT_INT) {
@@ -434,9 +462,6 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	      score = (uint32_t) (*vcfscore * 10000); // for scores in [0,1] 
 	      free(vcfscore);
 	    }
-	  } else {
-	    if (precise) score = srSupport * (uint32_t) (100 * srAlignQuality);
-	    else score = peSupport * (uint32_t) peMapQuality;
 	  }
 	  
 	  // Is this a selected interval
@@ -458,6 +483,9 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    if (bcf_get_info_int32(hdr[idx], rec[idx], "HOMLEN", &homlen, &nhomlen) > 0) homlenVal = *homlen;
 	    bcf_get_info_int32(hdr[idx], rec[idx], "CIPOS", &cipos, &ncipos);
 	    bcf_get_info_int32(hdr[idx], rec[idx], "CIEND", &ciend, &nciend);
+	    float srAlignQuality = 0;
+	    if (bcf_get_info_float(hdr[idx], rec[idx], "SRQ", &srq, &nsrq) > 0) srAlignQuality = *srq;
+
 	    std::string consensus;
 	    float ceVal = 0;
 	    if (precise) {
@@ -469,7 +497,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    // Create new record
 	    rout->rid = bcf_hdr_name2id(hdr_out, chrName.c_str());
 	    rout->pos = rec[idx]->pos;
-	    rout->qual = 0;
+	    rout->qual = score;
 	    std::string id;
 	    if (c.files.size() == 1) id = std::string(rec[idx]->d.id); // Within one VCF file IDs are unique
 	    else {
@@ -505,7 +533,9 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	      bcf_update_info_int32(hdr_out, rout, "INSLEN", &inslenVal, 1);
 	      bcf_update_info_int32(hdr_out, rout, "HOMLEN", &homlenVal, 1);
 	      bcf_update_info_int32(hdr_out, rout, "SR", &srSupport, 1);
-	      bcf_update_info_float(hdr_out, rout, "SRQ", &srAlignQuality, 1);	
+	      int32_t tmpi = srMapQuality;
+	      bcf_update_info_int32(hdr_out, rout, "SRMAPQ", &tmpi, 1);
+	      bcf_update_info_float(hdr_out, rout, "SRQ", &srAlignQuality, 1);
 	      bcf_update_info_string(hdr_out, rout, "CONSENSUS", consensus.c_str());
 	      bcf_update_info_float(hdr_out, rout, "CE", &ceVal, 1);
 	    }
@@ -532,6 +562,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   if (homlen != NULL) free(homlen);
   if (inslen != NULL) free(inslen);
   if (mapq != NULL) free(mapq);
+  if (srmapq != NULL) free(srmapq);
   if (ct != NULL) free(ct);
   if (srq != NULL) free(srq);
   if (svt != NULL) free(svt);
@@ -642,9 +673,8 @@ mergeRun(MergeConfig& c, int32_t const svt) {
   for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
     htsFile* ifile = bcf_open(c.files[file_c].string().c_str(), "r");
     bcf_hdr_t* hdr = bcf_hdr_read(ifile);
-    const char** seqnames = NULL;
     int nseq=0;
-    seqnames = bcf_hdr_seqnames(hdr, &nseq);
+    const char** seqnames = bcf_hdr_seqnames(hdr, &nseq);
     for(int32_t i = 0; i<nseq;++i) {
       std::string chrName(bcf_hdr_id2name(hdr, i));
       if (contigMap.find(chrName) == contigMap.end()) contigMap[chrName] = numseq++;
@@ -688,6 +718,9 @@ int merge(int argc, char **argv) {
   generic.add_options()
     ("help,?", "show help message")
     ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile)->default_value("sv.bcf"), "Merged SV BCF output file")
+    ("chunks,u", boost::program_options::value<uint32_t>(&c.chunksize)->default_value(500), "max. chunk size to merge groups of BCF files")
+    ("vaf,a", boost::program_options::value<float>(&c.vaf)->default_value(0.15), "min. fractional ALT support")
+    ("coverage,v", boost::program_options::value<uint32_t>(&c.coverage)->default_value(10), "min. coverage")
     ("minsize,m", boost::program_options::value<uint32_t>(&c.minsize)->default_value(0), "min. SV size")
     ("maxsize,n", boost::program_options::value<uint32_t>(&c.maxsize)->default_value(1000000), "max. SV size")
     ("precise,c", "Filter sites for PRECISE")
@@ -722,7 +755,7 @@ int merge(int argc, char **argv) {
   // Check command line arguments
   if ((vm.count("help")) || (!vm.count("input-file"))) { 
     std::cout << std::endl;
-    std::cout << "Usage: delly " << argv[0] << " [OPTIONS] <sample1.bcf> <sample2.bcf> ..." << std::endl;
+    std::cout << "Usage: delly " << argv[0] << " [OPTIONS] [<sample1.bcf> <sample2.bcf> ... | <list_of_bcf_files.tsv>]" << std::endl;
     std::cout << visible_options << "\n"; 
     return 0; 
   }
@@ -742,7 +775,32 @@ int merge(int argc, char **argv) {
   for(int i=0; i<argc; ++i) { std::cout << argv[i] << ' '; }
   std::cout << std::endl;
 
+  // Check chunksize
+  if (c.chunksize < 100) c.chunksize = 100;
+
   // Check input BCF files
+  if (c.files.size() == 1) {
+    // Assume the input file is a list
+    if (!(boost::filesystem::exists(c.files[0]) && boost::filesystem::is_regular_file(c.files[0]) && boost::filesystem::file_size(c.files[0]))) {
+      std::cerr << "Input file list " << c.files[0].string() << " is missing!" << std::endl;
+      return 1;
+    }
+    std::string fname = c.files[0].string();
+    c.files.clear();
+    std::ifstream lf(fname.c_str());
+    if (lf.good()) {
+      std::string line;
+      while(std::getline(lf, line)) {
+	if (!line.empty()) {
+	  if (line.at(line.length() - 1) == '\r' ) {
+	    line = line.substr(0, line.length() - 1);
+	  }
+	  c.files.push_back(line);
+	}
+      }
+      lf.close();
+    }
+  }
   for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
     htsFile* ifile = bcf_open(c.files[file_c].string().c_str(), "r");
     if (!ifile) {
@@ -751,25 +809,75 @@ int merge(int argc, char **argv) {
     }
     bcf_close(ifile);
   }
+
+  // Determine optimal chunksize
+  if (c.files.size() > c.chunksize) {
+    int32_t bestChunkSize = c.chunksize;
+    int32_t bestBinSize = 0;
+    for(uint32_t i = 50; i < c.chunksize; ++i) {
+      int32_t chunks = ((c.files.size() - 1) / i);
+      int32_t lastBin = c.files.size() - chunks * i;
+      if (lastBin > bestBinSize) {
+	bestBinSize = lastBin;
+	bestChunkSize = i;
+      }
+    }
+    c.chunksize = bestChunkSize;
+  }
   
   // Run merging
   boost::filesystem::path oldPath = c.outfile;
-  int rVal = 0;
-  int32_t maxSvt = 15;
-  std::vector<boost::filesystem::path> svtCollect(maxSvt);
-  for(int32_t svt = 0; svt < maxSvt; ++svt) {
-    svtCollect[svt] = boost::filesystem::unique_path();
-    c.outfile = svtCollect[svt];
-    rVal += mergeRun(c, svt);
+  std::vector<boost::filesystem::path> svtCollect(9);
+  for(int32_t svt = 0; svt < 9; ++svt) {
+    boost::uuids::uuid uuid = boost::uuids::random_generator()();
+    std::string filename = "svt" + boost::lexical_cast<std::string>(svt) + "_" + boost::lexical_cast<std::string>(uuid) + ".bcf";
+    svtCollect[svt] = filename;
+    if (c.files.size() <= c.chunksize) {
+      // Merge in one go
+      c.outfile = svtCollect[svt];
+      mergeRun(c, svt);
+    } else {
+      // Merge in chunks
+      std::vector<boost::filesystem::path> fileRestore = c.files;
+      uint32_t chunks = ((c.files.size() - 1) / c.chunksize) + 1;
+      std::vector<boost::filesystem::path> chunkCollect(chunks);
+      for(uint32_t ic = 0; ic < chunks; ++ic) {
+	boost::uuids::uuid uuid = boost::uuids::random_generator()();
+	std::string chunkfile = "chunk" + boost::lexical_cast<std::string>(ic) + "_" + boost::lexical_cast<std::string>(uuid) + ".bcf";
+	chunkCollect[ic] = chunkfile;
+	c.files.clear();
+	for(uint32_t k = ic * c.chunksize; ((k < ((ic+1) * c.chunksize)) && (k < fileRestore.size())); ++k) c.files.push_back(fileRestore[k]);
+	c.outfile = chunkCollect[ic];
+	mergeRun(c, svt);
+      }
+      // Merge chunks
+      c.files = chunkCollect;
+      c.outfile = svtCollect[svt];
+      // Reset VAF and coverage because these are site lists!
+      float vafStore = c.vaf;
+      uint32_t coverageStore = c.coverage;
+      c.vaf = 0;
+      c.coverage = 0;
+      mergeRun(c, svt);
+      c.vaf = vafStore;
+      c.coverage = coverageStore;
+      // Clean-up
+      for(uint32_t ic = 0; ic < chunks; ++ic) {
+	boost::filesystem::remove(chunkCollect[ic]);
+	boost::filesystem::remove(boost::filesystem::path(chunkCollect[ic].string() + ".csi"));
+      }
+      c.files = fileRestore;
+    }
   }
+  
   // Merge temporary files
   c.outfile = oldPath;
   mergeBCFs(c, svtCollect);
-  for(int32_t svt = 0; svt < maxSvt; ++svt) {
+  for(int32_t svt = 0; svt < 9; ++svt) {
     boost::filesystem::remove(svtCollect[svt]);
     boost::filesystem::remove(boost::filesystem::path(svtCollect[svt].string() + ".csi"));
   }
-  return rVal;
+  return 0;
 }
 
 }
