@@ -1,27 +1,8 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef MODVCF_H
 #define MODVCF_H
 
+#include <boost/progress.hpp>
+
 #include <htslib/sam.h>
 #include <htslib/vcf.h>
 
@@ -392,6 +373,148 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
 }
 
 
+// Long-reads
+template<typename TConfig>
+inline void
+outputVcf(TConfig const& c, std::vector<StructuralVariantRecord> const& sv)
+{
+  typedef std::vector<StructuralVariantRecord> TSVs;
+  
+  // Open one bam file header
+  samFile* samfile = sam_open(c.files[0].string().c_str(), "r");  
+  hts_set_fai_filename(samfile, c.genome.string().c_str());
+  bam_hdr_t* bamhd = sam_hdr_read(samfile);
+
+  // Output all structural variants
+  htsFile *fp = hts_open(c.outfile.string().c_str(), "wb");
+  bcf_hdr_t *hdr = bcf_hdr_init("w");
+
+  // Print vcf header
+  boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
+  boost::gregorian::date today = now.date();
+  std::string datestr("##fileDate=");
+  datestr += boost::gregorian::to_iso_string(today);
+  bcf_hdr_append(hdr, datestr.c_str());
+  bcf_hdr_append(hdr, "##ALT=<ID=DEL,Description=\"Deletion\">");
+  bcf_hdr_append(hdr, "##ALT=<ID=DUP,Description=\"Duplication\">");
+  bcf_hdr_append(hdr, "##ALT=<ID=INV,Description=\"Inversion\">");
+  bcf_hdr_append(hdr, "##ALT=<ID=BND,Description=\"Translocation\">");
+  bcf_hdr_append(hdr, "##ALT=<ID=INS,Description=\"Insertion\">");
+  bcf_hdr_append(hdr, "##FILTER=<ID=LowQual,Description=\"Long read support below 3.\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CIPOS,Number=2,Type=Integer,Description=\"Breakpoint confidence interval around POS\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CIEND,Number=2,Type=Integer,Description=\"Breakpoint confidence interval around END\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for END coordinate in case of a translocation\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=END,Number=1,Type=Integer,Description=\"End position of the structural variant\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"SV length\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Long read support\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=SRQ,Number=1,Type=Float,Description=\"Split-read consensus alignment quality\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=MAPQ,Number=1,Type=Integer,Description=\"Mean mapping quality of long reads\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CT,Number=1,Type=String,Description=\"Connection type\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=SVTYPE,Number=1,Type=String,Description=\"Type of structural variant\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=SVMETHOD,Number=1,Type=String,Description=\"Type of approach used to detect SV\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=INSLEN,Number=1,Type=Integer,Description=\"Predicted length of the insertion\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
+
+  // Add reference
+  std::string refloc("##reference=");
+  refloc += c.genome.string();
+  bcf_hdr_append(hdr, refloc.c_str());
+  for (int i = 0; i<bamhd->n_targets; ++i) {
+    std::string refname("##contig=<ID=");
+    refname += std::string(bamhd->target_name[i]) + ",length=" + boost::lexical_cast<std::string>(bamhd->target_len[i]) + ">";
+    bcf_hdr_append(hdr, refname.c_str());
+  }
+  // Add samples
+  bcf_hdr_add_sample(hdr, NULL);
+  bcf_hdr_write(fp, hdr);
+
+  if (!sv.empty()) {
+    // Quality threshold estimation
+    std::vector<float> qcf;
+    for(typename TSVs::const_iterator svIter = sv.begin(); svIter!=sv.end(); ++svIter) {
+      if (svIter->srSupport == 0) continue;
+      qcf.push_back((float) svIter->srSupport * (float) svIter->srAlignQuality);
+    }
+    std::sort(qcf.begin(), qcf.end());
+    float qualThreshold = qcf[qcf.size() * 0.05];
+    
+    // Iterate SVs
+    bcf1_t *rec = bcf_init();
+    for(typename TSVs::const_iterator svIter = sv.begin(); svIter!=sv.end(); ++svIter) {
+      if (svIter->srSupport == 0) continue;
+
+      // Output main vcf fields
+      float qcv = (float) svIter->srSupport * svIter->srAlignQuality;
+      int32_t tmpi = bcf_hdr_id2int(hdr, BCF_DT_ID, "PASS");
+      if (qcv < qualThreshold) tmpi = bcf_hdr_id2int(hdr, BCF_DT_ID, "LowQual");
+      rec->rid = bcf_hdr_name2id(hdr, bamhd->target_name[svIter->chr]);
+      rec->pos = svIter->svStart;
+      std::string id(_addID(svIter->svt));
+      std::string padNumber = boost::lexical_cast<std::string>(svIter->id);
+      padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
+      id += padNumber;
+      bcf_update_id(hdr, rec, id.c_str());
+      std::string alleles = "N,<";
+      alleles += _addID(svIter->svt);
+      alleles += ">";
+      bcf_update_alleles_str(hdr, rec, alleles.c_str());
+      bcf_update_filter(hdr, rec, &tmpi, 1);
+      
+      // Add INFO fields
+      bcf_update_info_string(hdr, rec, "SVTYPE", _addID(svIter->svt).c_str());
+      std::string dellyVersion("EMBL.DELLYv");
+      dellyVersion += dellyVersionNumber;
+      bcf_update_info_string(hdr,rec, "SVMETHOD", dellyVersion.c_str());
+      bcf_update_info_string(hdr,rec, "CHR2", bamhd->target_name[svIter->chr2]);
+      if (svIter->svt != 4) tmpi = svIter->svEnd;
+      else tmpi = svIter->svStart + 2;
+      bcf_update_info_int32(hdr, rec, "END", &tmpi, 1);
+      if (svIter->svt < DELLY_SVT_TRANS) {
+	if (svIter->svt != 4) tmpi = svIter->svEnd - svIter->svStart;
+	else tmpi = svIter->insLen;
+      } else tmpi = 0;
+      bcf_update_info_int32(hdr, rec, "SVLEN", &tmpi, 1);
+      bcf_update_info_string(hdr, rec, "CT", _addOrientation(svIter->svt).c_str());
+      int32_t ciend[2];
+      ciend[0] = svIter->ciendlow;
+      ciend[1] = svIter->ciendhigh;
+      int32_t cipos[2];
+      cipos[0] = -svIter->ciposlow;
+      cipos[1] = svIter->ciposhigh;
+      bcf_update_info_int32(hdr, rec, "CIPOS", cipos, 2);
+      bcf_update_info_int32(hdr, rec, "CIEND", ciend, 2);
+      tmpi = svIter->srSupport;
+      bcf_update_info_int32(hdr, rec, "SR", &tmpi, 1);
+      tmpi = (int32_t) (svIter->mapq / svIter->srSupport);
+      bcf_update_info_int32(hdr, rec, "MAPQ", &tmpi, 1);
+      tmpi = svIter->insLen;
+      bcf_update_info_int32(hdr, rec, "INSLEN", &tmpi, 1);
+      float tmpf = svIter->srAlignQuality;
+      bcf_update_info_float(hdr, rec, "SRQ", &tmpf, 1);
+      bcf_update_info_string(hdr, rec, "CONSENSUS", svIter->consensus.c_str());
+      tmpf = entropy(svIter->consensus);
+      bcf_update_info_float(hdr, rec, "CE", &tmpf, 1);
+      
+      rec->qual = svIter->mapq;
+      bcf_write1(fp, hdr, rec);
+      bcf_clear1(rec);
+    }
+    bcf_destroy1(rec);
+  }
+
+  // Close BAM file
+  bam_hdr_destroy(bamhd);
+  sam_close(samfile);
+
+  // Close VCF file
+  bcf_hdr_destroy(hdr);
+  hts_close(fp);
+
+  // Build index
+  bcf_index_build(c.outfile.string().c_str(), 14);
+}
+ 
 template<typename TConfig, typename TStructuralVariantRecord, typename TJunctionCountMap, typename TReadCountMap, typename TCountMap>
 inline void
 vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJunctionCountMap const& jctCountMap, TReadCountMap const& readCountMap, TCountMap const& spanCountMap)
