@@ -1,24 +1,3 @@
-/*
-============================================================================
-DELLY: Structural variant discovery by integrated PE mapping and SR analysis
-============================================================================
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-============================================================================
-Contact: Tobias Rausch (rausch@embl.de)
-============================================================================
-*/
-
 #ifndef SHORTPE_H
 #define SHORTPE_H
 
@@ -107,10 +86,9 @@ namespace torali
       if (srStore[refIndex].empty()) continue;
 
       // Load sequence
-      char* seq = NULL;
       int32_t seqlen = -1;
       std::string tname(hdr->target_name[refIndex]);
-      seq = faidx_fetch_seq(fai, tname.c_str(), 0, hdr->target_len[refIndex], &seqlen);
+      char* seq = faidx_fetch_seq(fai, tname.c_str(), 0, hdr->target_len[refIndex], &seqlen);
       
       // Collect all split-read pos
       typedef boost::dynamic_bitset<> TBitSet;
@@ -530,7 +508,7 @@ namespace torali
     
     // Augment PE SVs and append missing SR SVs
     for(int32_t svt = 0; svt < 10; ++svt) {
-      for(uint32_t i = 0; i < sr.size(); ++i) {
+      for(int32_t i = 0; i < (int32_t) sr.size(); ++i) {
 	if (sr[i].svt != svt) continue;
 	if ((sr[i].srSupport == 0) || (sr[i].srAlignQuality == 0)) continue; // SR assembly failed
 
@@ -565,7 +543,38 @@ namespace torali
 	}
 	
 	// SR only SV
-	if (!svExists) pe.push_back(sr[i]);
+	if (!svExists) {
+	  // Make sure there is no PRECISE duplicate
+	  int32_t precSearchWindow = 10;
+	  bool preciseDuplicate = false;
+	  for(int32_t j = i + 1; j < (int32_t) sr.size(); ++j) {
+	    if (std::abs(sr[i].svStart - sr[j].svStart) > precSearchWindow) break;
+	    if (sr[i].svt != sr[j].svt) continue;   // Mismatching SV types
+	    if ((sr[i].chr != sr[j].chr) || (sr[i].chr2 != sr[j].chr2)) continue;  // Mismatching chr
+
+	    // Breakpoints within PE confidence interval?
+	    if ((sr[j].svStart + sr[j].ciposlow <= sr[i].svStart) && (sr[i].svStart <= sr[j].svStart + sr[j].ciposhigh)) {
+	      if ((sr[j].svEnd + sr[j].ciendlow <= sr[i].svEnd) && (sr[i].svEnd <= sr[j].svEnd + sr[j].ciendhigh)) {
+		// Duplicate, keep better call
+		if ((sr[i].srSupport < sr[j].srSupport) || ((i < j) && (sr[i].srSupport == sr[j].srSupport))) preciseDuplicate = true;
+	      }
+	    }
+	  }
+	  for(int32_t j = i - 1; j>=0; --j) {
+	    if (std::abs(sr[i].svStart - sr[j].svStart) > precSearchWindow) break;
+	    if (sr[i].svt != sr[j].svt) continue;   // Mismatching SV types
+	    if ((sr[i].chr != sr[j].chr) || (sr[i].chr2 != sr[j].chr2)) continue;  // Mismatching chr
+
+	    // Breakpoints within PE confidence interval?
+	    if ((sr[j].svStart + sr[j].ciposlow < sr[i].svStart) && (sr[i].svStart < sr[j].svStart + sr[j].ciposhigh)) {
+	      if ((sr[j].svEnd + sr[j].ciendlow < sr[i].svEnd) && (sr[i].svEnd < sr[j].svEnd + sr[j].ciendhigh)) {
+		// Duplicate, keep better call
+		if ((sr[i].srSupport < sr[j].srSupport) || ((i < j) && (sr[i].srSupport == sr[j].srSupport))) preciseDuplicate = true;
+	      }
+	    }
+	  }
+	  if (!preciseDuplicate) pe.push_back(sr[i]);
+	}
       }
     }
   }
